// Java standard + interface graphique
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

// CORBA
import org.omg.CORBA.*;
import org.omg.CosNaming.*;

// Classes crees par idl (JacORB)
import Convertisseur.*;

public class Client extends JFrame {

	public static final boolean COM_ACTIVE = true;

	public static final String servantName = "Euro";
	public static final String contextName = "EURO_context";
	// Ne pas chosir un autre nom que "NameService"
	public static final String namingServiceName = "NameService";

	private Euro euro = null;
	
	private String devise = null;
	private double taux;
	private double deviseConv, euroConv;
	
	// Objets interface graphique
	private JPanel contentPane = null;

	// Panel contenant les panels reglages et conversion
	private JPanel centerPanel = null;

	// Panel reglages
	private JPanel settingsPanel = null;
	private JLabel deviseSettingsLabel = null;
	private JTextField deviseSettingsTField = null;
	private JLabel tauxSettingsLabel = null;
	private JTextField tauxSettingsTField = null;
	private JButton validateDeviseButton = null;
	private JButton validateTauxButton = null;
	
	// Panel conversion
	private JPanel convPanel = null;
	private JLabel deviseConvLabel = null;
	private JTextField deviseConvTField = null;
	private JButton toEuroButton = null;
	private JButton toDeviseButton = null;
	private JLabel euroConvLabel = null;
	private JTextField euroConvTField = null;
	
	// Panel contenant le bas de la frame
	private JPanel bottomPanel = null;
	private JButton quitButton = null;
	
	/**
	 * Constructeur de notre classe client
	 * @param args arguments de la ligne de commande
	 */
	public Client(String[] args) {
		devise = "?";
		taux = 0;
		deviseConv = 0;
		euroConv = 0;
		
		try {
			if (COM_ACTIVE) {
				initCom(args);
				initFields();
			}
			initInterface();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Initialisation de la communication avec le serveur C++
	 * @param args arguments de la ligne de commande
	 * @throws Exception
	 */
	private void initCom(String[] args) throws Exception {
		// Etape 1 : Initialisation de l'ORB
		// ORB.init() permet de se connecter a l'ORB
		ORB orb = ORB.init(args, null);

		// Etape 2 : Obtention d'un contexte racine associe au service de
		// nommage dont le nom est "NameService" (namingServiceName)
		org.omg.CORBA.Object rootNamingContextObj
			= orb.resolve_initial_references(namingServiceName);
		NamingContext rootNamingContext
			= NamingContextHelper.narrow(rootNamingContextObj);
		
		// Etape 3 : Recuperation d'une reference de notre objet "Euro"
		// (servantName) a partir du contexte racine et du contexte
		// "Euro_context" (contextName)
		// (voir fonction bindObjectToNamingService() dans server.cpp)
		NameComponent[] path = {
				new NameComponent(contextName, "Context"),
				new NameComponent(servantName, "Object")
		};
		org.omg.CORBA.Object euroObj = rootNamingContext.resolve(path);
		euro = EuroHelper.narrow(euroObj);
	}  // fin initCom()
	
	/**
	 * Initialisation des champs "devise" et "taux" en interrogeant le
	 * serveur C++ par l'intermediaire de notre objet distribue "Euro"
	 * (servant)
	 */
	private void initFields() {
		devise = euro.devise();
		taux = euro.taux();
	}
	
	/**
	 * Initialisation de l'interface graphique
	 */
	private void initInterface() {
		this.setSize(400, 300);
		this.setTitle("Convertisseur");
		this.setResizable(false);
		
	    contentPane = (JPanel)this.getContentPane();
	    contentPane.setLayout(new BorderLayout());
	    
	    centerPanel = new JPanel();
	    centerPanel.setLayout(new GridLayout(2, 1, 10, 10));

	    // Panel reglages
	    settingsPanel = new JPanel();
	    settingsPanel.setLayout(new GridBagLayout());
	    TitledBorder settingsPanelBorder
	    	= new TitledBorder(new EtchedBorder(), "Rglages");
	    settingsPanel.setBorder(settingsPanelBorder);
	    deviseSettingsLabel = new JLabel("Devise");
	    deviseSettingsTField = new JTextField(devise);
	    deviseSettingsTField.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		validateDeviseCB();
	    	}
	    });
	    validateDeviseButton = new JButton("Valider");
	    validateDeviseButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		validateDeviseCB();
	    	}
	    });
	    tauxSettingsLabel = new JLabel("Taux");
	    tauxSettingsTField = new JTextField("" + taux);
	    tauxSettingsTField.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		validateTauxCB();
	    	}
	    });
	    validateTauxButton = new JButton("Valider");
	    validateTauxButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		validateTauxCB();
	    	}
	    });
	    
	    // Layout panel reglages
	    // deviseSettingsLabel
	    GridBagConstraints constraints = new GridBagConstraints();	    
	    constraints.gridx = 0;
	    constraints.gridy = 0;
	    constraints.gridwidth = 1;
	    constraints.gridheight = 1;
	    constraints.weightx = 0;
	    constraints.weighty = 100;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.EAST;
	    constraints.insets = new Insets(5, 5, 5, 5); // top, left, bottom, right
	    constraints.ipadx = 0;
	    constraints.ipady = 0;
	    settingsPanel.add(deviseSettingsLabel, constraints);
	    
	    // deviseSettingsTField    
	    constraints.gridx = 1;
	    constraints.gridy = 0;
	    constraints.weightx = 100;
	    constraints.fill = GridBagConstraints.HORIZONTAL;
	    constraints.anchor = GridBagConstraints.CENTER;
	    settingsPanel.add(deviseSettingsTField, constraints);	
	    
	    // validateDeviseButton    
	    constraints.gridx = 2;
	    constraints.gridy = 0;
	    constraints.weightx = 0;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.CENTER;
	    settingsPanel.add(validateDeviseButton, constraints);	    

	    // tauxSettingsLabel   
	    constraints.gridx = 0;
	    constraints.gridy = 1;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.EAST;
	    settingsPanel.add(tauxSettingsLabel, constraints);
	    
	    // tauxSettingsTField    
	    constraints.gridx = 1;
	    constraints.gridy = 1;
	    constraints.fill = GridBagConstraints.HORIZONTAL;
	    constraints.anchor = GridBagConstraints.CENTER;
	    settingsPanel.add(tauxSettingsTField, constraints);	
	    
	    // validateTauxButton    
	    constraints.gridx = 2;
	    constraints.gridy = 1;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.CENTER;
	    settingsPanel.add(validateTauxButton, constraints);	    
	    
	    // Panel conversion
	    convPanel = new JPanel();
	    convPanel.setLayout(new GridBagLayout());
	    TitledBorder convPanelBorder
    		= new TitledBorder(new EtchedBorder(), "Conversion");
	    convPanel.setBorder(convPanelBorder);
	    deviseConvLabel = new JLabel(devise + " :");
	    deviseConvLabel.setMinimumSize(new Dimension(140, 25));
	    deviseConvTField = new JTextField("" + deviseConv);
	    deviseConvTField.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		deviseConvCB();
	    	}
	    });
	    toEuroButton = new JButton(">>");
	    toEuroButton.setToolTipText("Conversion " + devise + " > Euro");
	    toEuroButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		toEuroCB();
	    	}
	    });
	    toDeviseButton = new JButton("<<");
	    toDeviseButton.setToolTipText("Conversion Euro > " + devise);
	    toDeviseButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		toDeviseCB();
	    	}
	    });
	    euroConvLabel = new JLabel(" :");
	    euroConvLabel.setMinimumSize(new Dimension(140, 25));
	    euroConvTField = new JTextField("" + euroConv);	    
	    euroConvTField.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		euroConvCB();
	    	}
	    });
	    
	    // Layout panel conversion
	    // deviseConvLabel    
	    constraints.gridx = 0;
	    constraints.gridy = 0;
	    constraints.weightx = 0;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.WEST;
	    convPanel.add(deviseConvLabel, constraints);	
	    
	    // euroConvLabel    
	    constraints.gridx = 2;
	    constraints.gridy = 0;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.WEST;
	    convPanel.add(euroConvLabel, constraints);	    

	    // deviseConvTField   
	    constraints.gridx = 0;
	    constraints.gridy = 1;
	    constraints.weightx = 100;
	    constraints.gridheight = 2;
	    constraints.fill = GridBagConstraints.HORIZONTAL;
	    constraints.anchor = GridBagConstraints.CENTER;
	    convPanel.add(deviseConvTField, constraints);
	    
	    // toEuroButton    
	    constraints.gridx = 1;
	    constraints.gridy = 1;
	    constraints.weightx = 0;
	    constraints.gridheight = 1;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.CENTER;
	    convPanel.add(toEuroButton, constraints);	
	    
	    // euroConvTField   
	    constraints.gridx = 2;
	    constraints.gridy = 1;
	    constraints.weightx = 100;
	    constraints.gridheight = 2;
	    constraints.fill = GridBagConstraints.HORIZONTAL;
	    constraints.anchor = GridBagConstraints.CENTER;
	    convPanel.add(euroConvTField, constraints);   
	    
	    // toDeviseButton    
	    constraints.gridx = 1;
	    constraints.gridy = 2;
	    constraints.weightx = 0;
	    constraints.gridheight = 1;
	    constraints.fill = GridBagConstraints.NONE;
	    constraints.anchor = GridBagConstraints.CENTER;
	    convPanel.add(toDeviseButton, constraints);	    
	    
	    centerPanel.add(settingsPanel);
	    centerPanel.add(convPanel);
	    
	    // Panel du bas de la frame
	    bottomPanel = new JPanel();
	    bottomPanel.setLayout(new FlowLayout(SwingConstants.RIGHT));
	    quitButton = new JButton("Quitter");
	    quitButton.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent e) {
	    		closeApplicationCB();
	    	}
	    });
	    bottomPanel.add(quitButton);
		
	    contentPane.add(centerPanel, BorderLayout.CENTER);
	    contentPane.add(bottomPanel, BorderLayout.SOUTH);		

	    // Ecouteur d'evenements pour la fermeture de l'application
	    this.addWindowListener(new WindowAdapter() {
	      public void windowClosing(WindowEvent e) {
	        closeApplicationCB();
	      }
	    });
	}  // fin initInterface()
	
	private void validateDeviseCB() {
		if (COM_ACTIVE) {
			devise = deviseSettingsTField.getText();
			euro.devise(devise);
			devise = euro.devise();
		    toDeviseButton.setToolTipText("Conversion Euro > " + devise);
		    toEuroButton.setToolTipText("Conversion " + devise + " > Euro");
		}
		deviseSettingsTField.setText(devise);
		deviseConvLabel.setText(devise + " :");
	}
	
	private void validateTauxCB() {
		if (COM_ACTIVE) {
			taux = Double.parseDouble(tauxSettingsTField.getText());
			euro.taux(taux);
			taux = euro.taux();
		}	
		tauxSettingsTField.setText("" + taux);
	}
	
	private void toDeviseCB() {
		if (COM_ACTIVE) {
			euroConv = Double.parseDouble(euroConvTField.getText());
			deviseConv = euro.toDevise(euroConv);
			deviseConvTField.setText("" + deviseConv);
		}
	}
	
	private void toEuroCB() {
		if (COM_ACTIVE) {
			deviseConv = Double.parseDouble(deviseConvTField.getText());
			euroConv = euro.toEuro(deviseConv);			
			euroConvTField.setText("" + euroConv);
		}		
	}
	
	private void deviseConvCB() {
		deviseConv = Double.parseDouble(deviseConvTField.getText());
	}
	
	private void euroConvCB() {
		euroConv = Double.parseDouble(euroConvTField.getText());		
	}
	
	private void closeApplicationCB() {
		System.exit(0);
	}
	
	/**
	 * @param args arguments de la ligne de commande
	 */
	public static void main(String[] args) {
		Client client = new Client(args);
		
	    // Poitionnement de la frame principale au centre de l'ecran
	    Dimension frameSize = client.getSize();
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    if (frameSize.height > screenSize.height) {
	      frameSize.height = screenSize.height;
	    }
	    if (frameSize.width > screenSize.width) {
	      frameSize.width = screenSize.width;
	    }
	    client.setLocation((screenSize.width-frameSize.width)/2,
	    				   (screenSize.height-frameSize.height)/2);

		client.setVisible(true);
	}
}
